-- if os.getenv("LOCAL_LUA_DEBUGGER_VSCODE") == "1" then
--     require("lldebugger").start()
-- end

io.stdout:setvbuf('no')

local PDS = require("poisson")
local points
local oScreen
local scale = 1
local panX, panY = 0, 0

local function drawOffscreen()
    local settings = PDS.GetSettings()

    love.graphics.setCanvas(oScreen)
    love.graphics.clear()
    love.graphics.setColor(1, 0, 1, 1)

    love.graphics.line(panX * scale, 0, panX * scale, 600) -- Updated.
    love.graphics.line(0, panY * scale, 800, panY * scale) -- Updated.

    for _, pos in ipairs(points) do
        if pos.c then
            love.graphics.setColor(1, 1, 1, 1)
        end
        love.graphics.points((pos.x + panX) * scale, (pos.y + panY) * scale) -- Updated.
        love.graphics.setColor(1, 0, 1, 1)
    end
    love.graphics.setColor(1, 1, 1, 1)
    love.graphics.print(
        " Attempts/Sample " .. tostring(settings.maxAttemptsPerSample) ..
        "\n" .. " minRadius  " .. tostring(settings.minRadius) ..
        "\n" .. " maxRadius  " .. tostring(settings.maxRadius) ..
        "\n" .. " stepRadius " .. tostring(settings.stepRadius) ..
        "\n" .. " spawnRadFac " .. tostring(settings.spawnRadiusFactor) ..
        "\n" .. " spawnType  " .. tostring(settings.spawnType) ..
        "\n" .. " scale      " .. tostring(scale) ..
        "\n\n" .. " Right click to regenerate "
    )
    love.graphics.print(
        " Attempts/Sample " .. tostring(settings.maxAttemptsPerSample) ..
        "\n" .. " minRadius  " .. tostring(settings.minRadius) ..
        "\n" .. " maxRadius  " .. tostring(settings.maxRadius) ..
        "\n" .. " stepRadius " .. tostring(settings.stepRadius) ..
        "\n" .. " spawnRadFac " .. tostring(settings.spawnRadiusFactor) ..
        "\n" .. " spawnType  " .. tostring(settings.spawnType) ..
        "\n" .. " scale      " .. tostring(scale) ..
        "\n\n" .. " Right click to regenerate "
    )
    love.graphics.setCanvas()
end

function love.load()
    math.randomseed(os.time())

    oScreen = love.graphics.newCanvas(love.graphics.getDimensions())

    local settings = PDS.GetSettings()
    settings.sizeW = love.graphics.getWidth()
    settings.sizeH = love.graphics.getHeight()
    settings.minRadius = 10
    settings.maxRadius = 10
    PDS.ApplySettings(settings)

    points = PDS.Generate()
    print("Requested: " .. settings.maxResults, "Got: " .. #points)

    drawOffscreen()
end

function love.draw()
    love.graphics.draw(oScreen)
end

function love.mousepressed(x, y, button, istouch, presses)
    if button == 2 then
        local settings = PDS.GetSettings()
        settings.maxAttemptsPerSample = math.random(5, 100)
        settings.minRadius = math.random(5, 10)
        settings.maxRadius = math.random(10, 50)
        settings.stepRadius = math.random(5, 25)
        settings.spawnType = math.random(1, 7)
        settings.spawnRadiusFactor = math.random(1, 16)
        PDS.ApplySettings(settings)

        points = PDS.Generate()
        print("Requested: " .. settings.maxResults, "Got: " .. #points)
        scale = 1
        panX, panY = 0, 0

        drawOffscreen()
    end
end

function love.wheelmoved(_, dy)
    local mX, mY = love.mouse.getPosition()
    local oldMx  = mX / scale
    local oldMy  = mY / scale

    scale = scale * 1.3 ^ dy

    local newMx = mX / scale
    local newMy = mY / scale

    panX = panX - oldMx + newMx
    panY = panY - oldMy + newMy

    drawOffscreen()
end
